#!/bin/bash

RAMDISK=M:
INPUTFILE_ENCODE=$RAMDISK/input
OUTPUTFILE_ENCODE=$RAMDISK/encoded
INPUTFILE_DECODE=$OUTPUTFILE_ENCODE
OUTPUTFILE_DECODE=$RAMDISK/decoded
TESTOUTPUT=$RAMDISK/temp

# -- FLAC --
time_flac_encode(){
	CURRENT_EXT=".flac"
	./timer64.exe ./flac.exe $PRESET $INPUTFILE_ENCODE.wav -o $OUTPUTFILE_ENCODE$CURRENT_EXT --totally-silent > time
}
time_flac_decode(){
	./timer64.exe ./flac.exe -d $INPUTFILE_DECODE$CURRENT_EXT -o $OUTPUTFILE_DECODE.wav --totally-silent > time
}

# -- ALAC --
time_alac_encode(){
	CURRENT_EXT=".m4a"
	./timer64.exe ./refalac64.exe -A -s $PRESET $INPUTFILE_ENCODE.wav -o $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_alac_decode(){
	./timer64.exe ./refalac64.exe -D -s $INPUTFILE_DECODE$CURRENT_EXT -o $OUTPUTFILE_DECODE.wav > time
}

# -- WavPack --
time_wv_encode(){
	CURRENT_EXT=".wv"
	./timer64.exe ./wavpack.exe -z -q $PRESET $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_wv_decode(){
	./timer64.exe ./wvunpack.exe -z -q $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav > time
}

# -- TAK --
time_tak_encode(){
	CURRENT_EXT=".tak"
	./timer64.exe ./Takc.exe -e -silent $PRESET $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_tak_decode(){
	./timer64.exe ./Takc.exe -d -silent $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav > time
}

# -- OptimFROG --
time_ofr_encode(){
	CURRENT_EXT=".ofr"
	./timer64.exe ./ofr.exe --encode --silent --preset $PRESET $INPUTFILE_ENCODE.wav --output $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_ofr_decode(){
	./timer64.exe ./ofr.exe --decode --silent $INPUTFILE_DECODE$CURRENT_EXT --output $OUTPUTFILE_DECODE.wav > time	
}

# -- Monkey's --
time_ape_encode(){
	CURRENT_EXT=".ape"
	./timer64.exe ./MAC.exe $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT $PRESET 2>/dev/null > time
}
time_ape_decode(){
	./timer64.exe ./MAC.exe $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav -d 2>/dev/null > time
}

# -- SAC --
time_sac_encode(){
	CURRENT_EXT=".sac"
	./timer64.exe ./sac.exe $PRESET $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_sac_decode(){
	./timer64.exe ./sac.exe --decode $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav > time	
}

# -- WMA Lossless --
time_wma_encode(){
	CURRENT_EXT=".wma"
	./timer64.exe ./WMAEncode64.exe -s -c lsl $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_wma_decode(){
	if [ $BPS -eq 24 ]; then
		./timer64.exe ./ffmpeg.exe -hide_banner -v 0 -i $INPUTFILE_DECODE$CURRENT_EXT -c:a pcm_s24le $OUTPUTFILE_DECODE.wav > time
	else
		./timer64.exe ./ffmpeg.exe -hide_banner -v 0 -i $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav > time
	fi
}

# -- MLP --
time_mlp_encode(){
	CURRENT_EXT=".mlp"
	./timer64.exe ./ffmpeg.exe -hide_banner -v 0 -i $INPUTFILE_ENCODE.wav -strict -2 $OUTPUTFILE_ENCODE$CURRENT_EXT > time
}
time_mlp_decode(){
	./timer64.exe ./ffmpeg.exe -hide_banner -v 0 -i $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav > time
}

# -- TTA --
time_tta_encode(){
	CURRENT_EXT=".tta"
	./timer64.exe ./tta.exe -e $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT 2>/dev/null > time
}
time_tta_decode(){
	./timer64.exe ./tta.exe -d $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav 2>/dev/null > time
}

# -- Shorten --
time_shn_encode(){
	CURRENT_EXT=".shn"
	./timer64.exe ./shorten.exe $PRESET $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT 2>/dev/null > time
}
time_shn_decode(){
	./timer64.exe ./shorten.exe -x $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav 2>/dev/null > time
}

# -- ALS --
time_als_encode(){
	CURRENT_EXT=".als"
	./timer64.exe ./mp4alsRM23.exe $PRESET $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT 2>/dev/null > time
}
time_als_decode(){
	./timer64.exe ./mp4alsRM23.exe -x $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav 2>/dev/null > time
}

# -- 7z --
time_7z_encode(){
	CURRENT_EXT=".7z"
	./timer64.exe ./7z.exe a $OUTPUTFILE_ENCODE$CURRENT_EXT $INPUTFILE_ENCODE.wav $PRESET > time
}
time_7z_decode(){
	mv $INPUTFILE_ENCODE.wav $INPUTFILE_ENCODE.2.wav
	./timer64.exe ./7z.exe x $INPUTFILE_DECODE$CURRENT_EXT -o$RAMDISK > time
	mv $INPUTFILE_ENCODE.wav $OUTPUTFILE_DECODE.wav
	mv $INPUTFILE_ENCODE.2.wav $INPUTFILE_ENCODE.wav
}

# -- LA --
time_la_encode(){
	CURRENT_EXT=".la"
	./timer64.exe ./la.exe $PRESET $INPUTFILE_ENCODE.wav $OUTPUTFILE_ENCODE$CURRENT_EXT 2>/dev/null > time
}
time_la_decode(){
	./timer64.exe ./la.exe $INPUTFILE_DECODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav 2>/dev/null > time
}

parse_time (){
	CPUTIME=`tail -n 2 time | head -n 1 | awk '{print $4}'`
	CPUSPEED=`echo "scale=2; $FILE_SECONDS/$CPUTIME" | bc`
	REALTIME=`tail -n 1 time | awk '{print $4}'`	
	REALSPEED=`echo "scale=2; $FILE_SECONDS/$REALTIME" | bc`
	SPEEDS="$CPUSPEED,$REALSPEED"
}

process_combination (){
	SAMPLES=`./metaflac --show-total-samples "$2/$3"`
	BPS=`./metaflac --show-bps "$2/$3"`
	RAWSIZE=$((BPS*CHANNELS*SAMPLES/8))
	SAMPLERATE=`./metaflac --show-sample-rate "$2/$3"`
	FILE_SECONDS=`echo "scale=2; $SAMPLES/$SAMPLERATE" | bc`

	# Prepare file for encoding 
	./flac -df "$2/$3" -o $INPUTFILE_ENCODE.wav --totally-silent
	
	WAVSIZE=`stat -c %s $INPUTFILE_ENCODE.wav`
	
	# skip WAVE header for MD5sum
	INPUTCMPSKIP=$((WAVSIZE-RAWSIZE))
	MD5INPUT=`tail -c +$INPUTCMPSKIP $INPUTFILE_ENCODE.wav | md5sum`
	
	# Time encode
	ENCODEFUNC="time_$1_encode"
	$ENCODEFUNC
	parse_time
	ENCODESPEED=$SPEEDS

	# Check compression
	COMPRSIZE=`stat -c %s $OUTPUTFILE_ENCODE$CURRENT_EXT`
	COMPRESSION=`echo "scale=5; $COMPRSIZE/$WAVSIZE" | bc`

	# Remove input to conserve RAM
	rm $INPUTFILE_ENCODE.wav

	# Time decode
	DECODEFUNC="time_$1_decode"
	$DECODEFUNC
	parse_time
	DECODESPEED=$SPEEDS
	
	DECWAVSIZE=`stat -c %s $OUTPUTFILE_DECODE.wav`
	
	OUTPUTCMPSKIP=$((DECWAVSIZE-RAWSIZE))
	MD5OUTPUT=`tail -c +$OUTPUTCMPSKIP $OUTPUTFILE_DECODE.wav | md5sum`
	
	if [ "$MD5OUTPUT" = "$MD5INPUT" ]; then LOSSLESS="lossless"; else LOSSLESS="not lossless"; fi

	rm $OUTPUTFILE_ENCODE$CURRENT_EXT $OUTPUTFILE_DECODE.wav
	
	OUTPUTLINE="${2#input/},${3//,/},$1,$PRESET,$ENCODESPEED,$DECODESPEED,$COMPRESSION,$LOSSLESS"

	echo $OUTPUTLINE >> $TESTOUTPUT
}

rm $RAMDISK/*
touch $RAMDISK/touch
touch $TESTOUTPUT
if [ -f output/"${1#input/}".csv ]
then
	read -p "Remove old results? (y/n) " -n 1 -r
	echo 
	if [[ $REPLY =~ ^[Yy]$ ]]
	then
		rm output/"${1#input/}".csv
	fi
fi

touch output/"${1#input/}".csv

if [ ! -f output/"${1#input/}".csv ]
then
	echo "Couldn't create output"
	exit
fi

I=0;
for FILE in "$1"/*.flac
do
	if [ ! -z $2 ]; then
		I=$((I+1))
		if [ $2 -gt $I ]; then
			continue
		fi
	fi
	rm $TESTOUTPUT
	touch $TESTOUTPUT
	
	FILENAME=${FILE#"$1/"}
	echo $FILENAME
	
	CHANNELS=`./metaflac --show-channels "$1/$FILENAME"`
	BPS=`./metaflac --show-bps "$1/$FILENAME"`
	SAMPLERATE=`./metaflac --show-sample-rate "$1/$FILENAME"`
	
	echo FLAC
	if [ $CHANNELS -eq 2 ]; then
		for PRESET in "-0" "-1" "-2" "-3" "-4" "-5" "-6" "-7" "-8"
		do
			process_combination flac "$1" "$FILENAME"
		done
	else
		for PRESET in "-0" "-3" "-5" "-6" "-7" "-8"
		do
			process_combination flac "$1" "$FILENAME"
		done
	fi

	if [ $CHANNELS -le 6 ]; then
		echo ALAC
		if [ $CHANNELS -eq 2 ]; then
			for PRESET in "--fast" ""
			do
				process_combination alac "$1" "$FILENAME"
			done
		else
			PRESET=""
			process_combination alac "$1" "$FILENAME"
		fi
	fi

	echo WavPack
	for PRESET in "-f" "" "-h" "-hh" "-x4f" "-x4" "-x4h" "-x4hh"
	do
		process_combination wv "$1" "$FILENAME"
	done

	if [ $CHANNELS -le 6 ] && [ $SAMPLERATE -le 192000 ]; then
		echo TAK
		for PRESET in "-p0" "-p0e" "-p0m" "-p1" "-p1e" "-p1m" "-p2" "-p2e" "-p2m" "-p3" "-p3e" "-p3m" "-p4" "-p4e" "-p4m"
		do
			process_combination tak "$1" "$FILENAME"
		done
	fi
	
	if [ $CHANNELS -le 2 ]; then
		echo OptimFROG
		for PRESET in "0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "max"
		do
			process_combination ofr "$1" "$FILENAME"
		done
	fi
	
	echo APE
	for PRESET in "-c1000" "-c2000" "-c3000" "-c4000" "-c5000"
	do	
		process_combination ape "$1" "$FILENAME"
	done

	if [ $CHANNELS -le 2 ] && [ $BPS -eq 16 ]; then
		echo LA
		for PRESET in "" "-high"
		do	
			process_combination la "$1" "$FILENAME"
		done	
	fi
	
	echo ALS
	for PRESET in "-a -o5" "-a" "-a -o20" "-a -o40" "-a -b" "-a -b -o20" "-a -b -o40" "-a -b -o1023" "-7"
	do
		process_combination als "$1" "$FILENAME"
	done
	
	# echo SAC
	# for PRESET in "" "--high" "--high --optimize=high --sparse-pcm"
	# do
		# process_combination sac "$1" "$FILENAME"
	# done	
	
	if [ $CHANNELS -le 2 ] && [ $BPS -eq 16 ]; then
		PRESET=""
		echo Shorten
		process_combination shn "$1" "$FILENAME"
	fi
	
	if [ $CHANNELS -le 6 ] && [ $SAMPLERATE -le 96000 ]; then
		PRESET=""
		echo WMA
		process_combination wma "$1" "$FILENAME"
	fi

	if [ $CHANNELS -le 6 ]; then	
		PRESET=""
		echo TTA
		process_combination tta "$1" "$FILENAME"
	fi
	
	# PRESET=""
	# echo MLP
	# process_combination mlp "$1" "$FILENAME"
	
	# PRESET=""
	# echo "7z"
	# process_combination "7z" "$1" "$FILENAME"
	
	cat $TESTOUTPUT >> output/"${1#input/}".csv
done
