#!/bin/bash

FILE=$1
NAME=${FILE%.csv}
STEMNAME=`echo $NAME | awk -F"/" '{print $NF}'`

#### ---- First assign colors, markers and names to codecs ---- ####

codec_name_translation_flac="FLAC 1.3.4";
codec_name_translation_wv="WavPack 5.4.0";
codec_name_translation_wvx4="WavPack 5.4.0, -x4";
codec_name_translation_tak="TAK 2.3.1";
codec_name_translation_ofr="OptimFROG v5.100";
codec_name_translation_la="LA 0.4b";
codec_name_translation_ape="Monkey's Audio 7.58";
codec_name_translation_shn="Shorten 3.6.1";
codec_name_translation_tta="TTA 2.3";
codec_name_translation_als="MP4 ALS RM23";
codec_name_translation_wma="WMA Lossless V12";
codec_name_translation_alac="ALAC (refalac 1.73)";

# Get all used coders
codecs_used=`awk -F ',' '{print $3}' "$FILE" | awk '!x[$0]++'`
codec_color_marker_unused=""


codec_color_marker_flac="darkblue ftriangle"
if [ -z "`echo "$codecs_used" | grep flac`" ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\ndarkblue ftriangle"`
fi

codec_color_marker_alac="red fsquare"
if [ -z `echo "$codecs_used" | grep alac` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\nred fsquare"`
fi

codec_color_marker_wv="dodgerblue fdiamond"
if [ -z `echo "$codecs_used" | grep wv` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\ndodgerblue fdiamond"`
fi

codec_color_marker_tak="green fcircle"
if [ -z `echo "$codecs_used" | grep tak` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\ngreen fcircle"`
fi

codec_color_marker_ofr="magenta triangle"
if [ -z `echo "$codecs_used" | grep -x ofr` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\nmagenta triangle"`
fi

codec_color_marker_ape="grey square"
if [ -x `echo "$codecs_used" | grep -x ape` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\ngrey square"`
fi

codec_color_marker_la="black diamond"
if [ -z `echo "$codecs_used" | grep -x la` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\nblack diamond"`
fi

codec_color_marker_als="olive circle"
if [ -z `echo "$codecs_used" | grep -x als` ]; then
	codec_color_marker_unused=`printf "$codec_color_marker_unused\nolive circle"`
fi

codec_color_marker_wvx4="brown cross"

codec_color_marker_shn="darkblue triangle"
codec_color_marker_wma="green square"
codec_color_marker_tta="red diamond"

# Remove empty line
codec_color_marker_unused=`printf "$codec_color_marker_unused\norchid asterisk" | tail -n +2`

if [ ! -z "`grep "wv,-x4" "$FILE"`" ]; then
	codecs_used_wvx4=`echo "$codecs_used" | sed 's/wv/wv\nwvx4/g'`
else
	codecs_used_wvx4=`echo "$codecs_used"`
fi

num_codecs=`echo "$codecs_used_wvx4" | wc -l`
num_separator=$(((num_codecs+2)/3))

# Parse options
if [ -z $2 ]; then
	MODE="default"
else
	MODE=$2
fi

#run_gle($codecdata,$basename,$setname,$title,$xtitle,$ytitle){
create_glefile(){
  # global $codec_name_translation, $color, $marker, $graphsize, $fontsize, $plot_big;
  tee temp/glefile <<EOF >/dev/null
papersize a4paper
margins 1.5 1.5 1.5 1.5

set just center
amove pagewidth()/2 pageheight()-1
set hei 0.75
text {\bf $setname}
amove pagewidth()/2 0
set hei 0.4
text These graphs are part of a report which can be found on
rmove 0 -0.4
text audiograaf.nl/losslesstest/Lossless audio codec comparison - revision 5 - cdda.html


amove 0 (pageheight()-6)/2+0.5
begin graph
	size pagewidth() (pageheight()-6)/2 
	hscale auto
	set hei 0.4
	title "{\bf Encoding} CPU-usage versus file size" dist 0.8 hei 0.5
	xtitle "CPU-usage, % of 1 core (smaller is better)" hei 0.45
	ytitle "File size, % of WAV-size (smaller is better)" hei 0.45
	xlabels hei 0.55
	ylabels hei 0.55
	axis grid base 0.5
	ticks color grey10
	
	xaxis log
	xlabels log l25b
	xaxis format "round 2 nozeroes append %"
	yaxis format "round 3 nozeroes append %"
EOF

    echo "$codecs_used_wvx4" | while read CODEC; do
		echo "    data \"data-$CODEC-encode\"" >> temp/glefile
	done

    I=1
    echo "$codecs_used_wvx4" | while read CODEC; do
		eval "codec_color_marker=\"\${codec_color_marker_$CODEC}\""
		codec_color=`echo $codec_color_marker | awk '{print $1}'`
		codec_marker=`echo $codec_color_marker | awk '{print $2}'`
	    echo -n "    d$I line marker $codec_marker color $codec_color" >> temp/glefile;

		if [ -z "`echo "$codec_marker" | grep "diamond"`" ]; then
			echo " msize 0.2" >> temp/glefile
		else
			echo " msize 0.27" >> temp/glefile
		fi
		I=$((I+1))
	done

    echo "end graph" >> temp/glefile
	
  tee -a temp/glefile <<EOF >/dev/null
amove 0 0.5
begin graph
	size pagewidth() (pageheight()-6)/2 
	hscale auto
	set hei 0.4
	title "{\bf Decoding} CPU-usage versus file size" dist 0.8 hei 0.5
	xtitle "CPU-usage, % of 1 core (smaller is better)" hei 0.45
	ytitle "File size, % of WAV-size (smaller is better)" hei 0.45
	xlabels hei 0.55
	ylabels hei 0.55
	axis grid base 0.5
	ticks color grey10
	
	xaxis log
	xlabels log l25b
	xaxis format "round 2 nozeroes append %"
	yaxis format "round 3 nozeroes append %"
EOF

    echo "$codecs_used_wvx4" | while read CODEC; do
		echo "    data \"data-$CODEC-decode\"" >> temp/glefile
	done

    I=1
    echo "$codecs_used_wvx4" | while read CODEC; do
		eval "codec_color_marker=\"\${codec_color_marker_$CODEC}\""
		codec_color=`echo $codec_color_marker | awk '{print $1}'`
		codec_marker=`echo $codec_color_marker | awk '{print $2}'`
	    echo -n "    d$I line marker $codec_marker color $codec_color" >> temp/glefile;

		if [ -z "`echo "$codec_marker" | grep "diamond"`" ]; then
			echo " msize 0.2" >> temp/glefile
		else
			echo " msize 0.27" >> temp/glefile
		fi
		I=$((I+1))
	done

    echo "end graph" >> temp/glefile

	tee -a temp/glefile <<EOF >/dev/null
begin key
    justify bc
	absolute pagewidth()/2 pageheight()-4
    compact
	boxcolor rgba255(255,255,255,128)
EOF

	I=1
    echo "$codecs_used_wvx4" | while read CODEC; do
		nr_presets_this_codec=`grep ",$CODEC," temp/results_per_album | awk -F ',' '{print $3}' | awk '!x[$0]++' | wc -l`
		eval "codec_color_marker=\"\${codec_color_marker_$CODEC}\""
		eval "codec_name_translation=\"\${codec_name_translation_$CODEC}\""
		codec_color=`echo $codec_color_marker | awk '{print $1}'`
		codec_marker=`echo $codec_color_marker | awk '{print $2}'`
		if [ $nr_presets_this_codec -eq 1 ]; then
			echo "    color $codec_color marker $codec_marker text \"$codec_name_translation\"" >> temp/glefile;
		else
			echo "    line color $codec_color marker $codec_marker text \"$codec_name_translation\"" >> temp/glefile;
		fi
		if [ $I -eq $num_separator ]; then
			echo "    separator" >> temp/glefile;
			I=1
		else
			I=$((I+1))
		fi
	done
	echo "end key">>temp/glefile
}


# if [ -z `echo "$MODE" | grep "noperc"` ];     then plot_inverted=1; else plot_inverted=0; fi
# if [ -z `echo "$MODE" | grep "track"` ];      then per_track=0;     else per_track=1; fi
# if [ -z `echo "$MODE" | grep "album"` ];      then per_album=0;     else per_album=1; fi
# if [ -z `echo "$MODE" | grep "onlyencode"` ]; then only_encode=0;   else only_encode=1; fi
# if [ -z `echo "$MODE" | grep "realtime"` ];   then realtime=0;      else realtime=1  ; fi

# List all albums
albums_used=`awk -F ',' '{print $1}' "$FILE" | awk '!x[$0]++'`
echo -n "" > temp/results_per_album
K=0

while read ALBUM; do
	rm temp/data-*-??code
	setname=`echo $ALBUM | awk -F"/" '{print $NF}'`
	echo "$setname"
	data_this_album=`grep "$ALBUM," "$FILE"`
	echo "$codecs_used" | while read CODEC; do
		presets_this_album_and_codec=`echo "$data_this_album" | sed 's/,alac,,/,alac, ,/' | grep ",$CODEC," | awk -F ',' '{print $4}' | awk '!x[$0]++'`
		echo "$presets_this_album_and_codec" | while read "PRESET"; do
			if [ -z `echo "$CODEC,$PRESET" | grep "wv,-x4"` ]; then
				CODEC_PRINT=$CODEC
			else
				CODEC_PRINT="wvx4"
			fi
			(echo -n $ALBUM,$CODEC_PRINT,$PRESET,; echo "$data_this_album" | grep ",$CODEC,$PRESET," | awk -F "," '{a+=$5; b+=$6; c+=$7; d+=$8; e+=$9; n+=1}END{printf "%.2f,%.2f,%.2f,%.2f,%.4f\n",a/n,b/n,c/n,d/n,e/n}') >> temp/results_per_album
			(echo "$data_this_album" | grep ",$CODEC,$PRESET," | awk -F "," '{a+=$5; e+=$9; n+=1}END{printf "%.2f,%.2f\n",n/a*100,e*100/n}') >> temp/data-$CODEC_PRINT-encode
			(echo "$data_this_album" | grep ",$CODEC,$PRESET," | awk -F "," '{c+=$7; e+=$9; n+=1}END{printf "%.2f,%.2f\n",n/c*100,e*100/n}') >> temp/data-$CODEC_PRINT-decode
		done
	done
	
	K=$((K+1))

	create_glefile
	gle -device pdf -output temp/graph.pdf temp/glefile
	gs -sDEVICE=pdfwrite -o "output/$ALBUM.pdf" -f temp/graph.pdf -c "[ /Title ($setname) /Author (Martijn van Beurden) /Subject (Lossless Audio Codec Comparison Revision 5  - see audiograaf.nl/losslesstest/Lossless audio codec comparison - revision 5 - cdda.html) /DOCINFO pdfmark" > /dev/null
done <<EOF
$albums_used
EOF

if [ $K -gt 1 ]; then

	# Now average over all albums
	echo -n "" > temp/total_results
	echo Totaal
	echo "$codecs_used_wvx4" | while read CODEC; do
		presets_this_codec=`sed 's/,alac,,/,alac, ,/' temp/results_per_album | grep ",$CODEC," | awk -F ',' '{print $3}' | awk '!x[$0]++'`
		echo -n "" > temp/data-$CODEC-encode
		echo -n "" > temp/data-$CODEC-decode
		echo "$presets_this_codec" | while read PRESET; do
			(echo -n $CODEC,$PRESET,; grep ",$CODEC,$PRESET," temp/results_per_album | awk -F "," '{a+=$4; b+=$5; c+=$6; d+=$7; e+=$8; n+=1}END{printf "%.2f,%.2f,%.2f,%.2f,%.4f\n",a/n,b/n,c/n,d/n,e/n}') >> temp/total_results
			(grep ",$CODEC,$PRESET," temp/results_per_album | awk -F "," '{a+=$4; e+=$8; n+=1}END{printf "%.2f,%.2f\n",n/a*100,e*100/n}') >> temp/data-$CODEC-encode
			(grep ",$CODEC,$PRESET," temp/results_per_album | awk -F "," '{c+=$6; e+=$8; n+=1}END{printf "%.2f,%.2f\n",n/c*100,e*100/n}') >> temp/data-$CODEC-decode
		done
	done
	
	setname=$STEMNAME
	create_glefile
	gle -device pdf -output temp/graph.pdf temp/glefile
	gs -sDEVICE=pdfwrite -o "$NAME.pdf" -f temp/graph.pdf -c "[ /Title ($setname) /Author (Martijn van Beurden) /Subject (Lossless Audio Codec Comparison Revision 5 - see audiograaf.nl/losslesstest/Lossless audio codec comparison - revision 5 - cdda.html) /DOCINFO pdfmark"  > /dev/null
	cp temp/results_per_album "$NAME split by source.csv"
fi
